const  dataSource = 'https://gist.githubusercontent.com/Razpudding/3711f9680fb0cbb38f7daec18e7ee631/raw/494905ee1a2f4b556dbc5e460442aeac053f708a/etnisch_profileren_2019_json'

const width = 600
const height = 600

const svg = d3.select('svg')
console.log(svg)
svg.attr('width', width + "px")
svg.attr('height', height + "px")
const x = d3.scaleLinear()
const y = d3.scaleLinear()
const color = d3.scaleOrdinal(d3.schemeDark2)

d3.json(dataSource)
	.then(json => {
  	console.log("starting with data length", json.length)
  	const filtered = json.filter(item => checkInvalid(item))
    console.log("data length after filtering 99999", filtered.length)
    const data = filtered.map(result => {
      return {
        id: result.response_ID,
        afkomst: result.Herkomst_def,
        totstand: result.Totstand == "De politie kwam naar mij toe" ? 1 : 0,
        contact: result.Contact_gehad,
        freqcontact: result.freqcontact,
        rapportcijfer: result.rapportcijfer
    	}
		})
    const transformed = transformData(data)
    console.log(transformed)

  	addScales(transformed)
  	drawScatterPlot(transformed)
  })

function transformData(data){
	const transformed = d3.nest()
  	//first group all respondents by afkomst
    .key(d => d.afkomst)
    .rollup(d => {
      return {
      	amount: d.length,
        benaderdTotaal: d.filter(respondent => respondent.totstand == 1).length,
        nietBenaderdTotaal: d.filter(respondent => respondent.totstand == 0).length,
				//Create an array of 0s for each rapportcijfer percentage we will fill in later
        rapportCijfersBenaderd: [0,0,0,0,0,0,0,0,0,0],
        rapportCijfersNietBenaderd: [0,0,0,0,0,0,0,0,0,0],
      }
  	})
  	.entries(data)
  //For each respondent, check if they approached the police and increment the amount of respondents that
  // had a certain rapportcijfer
  data.forEach(respondent => {
  	const land = transformed.find(afkomst => afkomst.key == respondent.afkomst)
    //console.log(land)
    if (respondent.totstand == 1){
      land.value.rapportCijfersBenaderd[respondent.rapportcijfer - 1] ++
    } else if (respondent.totstand == 0){
      land.value.rapportCijfersNietBenaderd[respondent.rapportcijfer - 1] ++
    }		
  })
  //For each country, convert the amount of people that had a certain rapportcijfer to a percentage
  transformed.forEach(land => {
    // console.log(land.value.rapportCijfersBenaderd)
    land.value.rapportCijfersBenaderd.forEach( (cijfer, index, array) => {
      array[index] = Math.round((cijfer / land.value.benaderdTotaal) * 100)
    })
    land.value.rapportCijfersNietBenaderd.forEach( (cijfer, index, array) => {
      array[index] = Math.round((cijfer / land.value.nietBenaderdTotaal) * 100)
    })
  })
  return transformed
}

//This function removes any items that have 99999 as an answer to any of the questions we're using for this viz!
function checkInvalid(item){
  const importantKeys = ["Herkomst_def", "Totstand", "Contact_gehad", "freqcontact" ,"rapportcijfer"]
	for (let key of importantKeys) {
  	if (item[key] == '99999') {
      // console.log("removing invalid item because", key)
    	return false
    }
  }
  return true
}
function drawScatterPlot(data){  
  data.forEach( (country, index) => {
    console.log(country)
    //Add all the circles for approached by police
  	let circlesTop = svg.selectAll(".top")
      .data(country.value.rapportCijfersBenaderd)
      .enter()
      .append("circle")
        .attr("class", country.key)
        .attr("r", d => circleSize(d))
        .attr("cx", (d, i) => x(i + 1))
        .attr("cy", (10 * index) + 100)
    		.attr("fill", color(index))
    
    //Add all the circles for approached police themselves
    let circlesBottom = svg.selectAll(".bottom")
      .data(country.value.rapportCijfersNietBenaderd)
      .enter()
      .append("circle")
        .attr("class", country.key)
        .attr("r", d => circleSize(d))
        .attr("cx", (d, i) => x(i + 1))
        .attr("cy", (10 * index) + 300)
    		.attr("fill", color(index))
  })
}

function addScales(data){
 	x.domain([1,10])	//From lowest to highest cijfer
  	.range([0, width])
  y.domain([0,255])
  	//By putting height first, we're fixing the inverse effect of y
  	.range([height,0])
}

//Determine the circlesize. I've tweaked this by hand  but there are nicer ways
function circleSize(item){
	if (item == 0){
   return 0.5
  }
  else if (item < 5){
   return 2 
  }
  else if (item < 15){
   return 3 
  }
  else if (item < 25){
   return 4 
  }
  else if (item < 35){
   return 5 
  }
  else if (item < 45){
   return 6 
  }
  else return 100
}